'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createLeakErrorFactory;

var _leftPad = require('left-pad');

var _leftPad2 = _interopRequireDefault(_leftPad);

var _prettyBytes = require('pretty-bytes');

var _prettyBytes2 = _interopRequireDefault(_prettyBytes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createLeakErrorFactory(iterationCount, totalHeapDiffCount) {
  var iterationsPerHeapDiff = iterationCount / totalHeapDiffCount;

  // `3` => `3`, `2.1` => `~2`, `4.7` => `~5`
  var formatInteger = function formatInteger(value) {
    return Math.round(value) !== value ? '~' + Math.round(value) : value;
  };

  /**
   * @param {memwatch.HeapDiff[]} subsequentHeapGrowths
   * @return {Error}
   */
  return function (subsequentHeapGrowths) {
    var lastHeapDiff = subsequentHeapGrowths[subsequentHeapGrowths.length - 1];
    var heapGrowthIterations = Math.round(subsequentHeapGrowths.length * iterationsPerHeapDiff);

    var growthInBytes = subsequentHeapGrowths.map(function (heapDiff) {
      return heapDiff.change.size_bytes;
    }).reduce(function (total, heapGrowth) {
      return total + heapGrowth;
    }, 0);

    return new Error(`Heap grew on ${subsequentHeapGrowths.length} subsequent garbage collections ` + `(${formatInteger(heapGrowthIterations)} of ${iterationCount} iterations) ` + `by ${(0, _prettyBytes2.default)(growthInBytes)}.\n\n` + `  Iterations between GCs: ${formatInteger(iterationsPerHeapDiff)}\n\n` + `  Final GC details:\n` + `  ${prettyHeapContents(lastHeapDiff).trimLeft()}\n`);
  };
}

function prettyHeapContents(lastHeapDiff) {
  var byGrowth = function byGrowth(a, b) {
    return a.size_bytes < b.size_bytes ? 1 : -1;
  };

  var formatHeapContent = function formatHeapContent(item) {
    return `[${(0, _leftPad2.default)((0, _prettyBytes2.default)(item.size_bytes), 10)}] [+ ${(0, _leftPad2.default)(item['+'], 3)}x] [- ${(0, _leftPad2.default)(item['-'], 3)}x] ${item.what}`;
  };

  var sortedDetails = [].concat(lastHeapDiff.change.details).sort(byGrowth);
  var formattedHeapContents = sortedDetails.map(function (heapContentItem) {
    return formatHeapContent(heapContentItem);
  });

  var heapContentLines = formattedHeapContents.length > 4 ? formattedHeapContents.slice(0, 4).concat(`... (${formattedHeapContents.length - 4} more)`) : formattedHeapContents;

  return heapContentLines.map(function (line) {
    return `  ${line}`;
  }).join('\n');
}